@@ -4,11 +4,12 @@
 //!
 //! ```
 //! use four_bar::{FourBar, Mechanism};
+//! use std::f64::consts::TAU;
 //!
 //! // A four-bar mechanism example
-//! let m = Mechanism::four_bar(&FourBar::default());
+//! let m = Mechanism::four_bar(&FourBar::example());
 //! // Get the trajectory of the coupler point
-//! let path = m.four_bar_loop(0., 360);
+//! let path = m.four_bar_loop(0., TAU, 360);
 //! ```
 #![cfg_attr(doc_cfg, feature(doc_cfg))]
 #![warn(missing_docs)]
@@ -16,7 +17,7 @@ pub use crate::{four_bar::*, mechanism::*, point::*};
 
 mod four_bar;
 mod mechanism;
-#[cfg(feature = "plotters")]
+#[cfg(feature = "plot")]
 pub mod plot;
 mod point;
 pub mod synthesis;
